#include "AppState.h"

#include <fmt/format.h>
#include <spdlog/spdlog.h>

#include <fcntl.h>
#include <optional>
extern "C" {
#include <i2c/smbus.h>
}
#include <linux/i2c-dev.h>
#include <sys/ioctl.h>

#include <cstdint>
#include <stdexcept>
#include <unistd.h>

std::ostream& operator<<(std::ostream& os, AppView view) {
  switch (view) {
  case AppView::MISSION_SELECT:
    return os << "MISSION_SELECT";
  case AppView::AMI:
    return os << "AMI";
  case AppView::DRIVER:
    return os << "DRIVER";
  case AppView::TESTING:
    return os << "TESTING";
  default:
    return os << "UNKNOWN_VIEW[" << static_cast<int>(view) << "]";
  }
}

AppState::AppState(const std::string& i2c_dev_path)
    : view{AppView::MISSION_SELECT} {
  i2c_dev_file = open(i2c_dev_path.c_str(), O_RDWR);
  if (i2c_dev_file < 0) {
    spdlog::error("Couldn't open I2C device: {}", errno);
    i2c_dev_file = std::nullopt;
    return;
  }

  if (ioctl(*i2c_dev_file, I2C_SLAVE, I2C_SLAVE_ADDR) < 0) {
    spdlog::error("Couldn't set I2C slave address: {}", errno);
    close(*i2c_dev_file);
    i2c_dev_file = std::nullopt;
  }
}

AppState::~AppState() {
  if (i2c_dev_file) {
    close(*i2c_dev_file);
  }
}

void AppState::poll() {
  if (!i2c_dev_file) {
    return;
  }

  uint8_t data[32];
  int count = i2c_smbus_read_block_data(*i2c_dev_file, I2C_POLL_COMMAND, data);
  if (count < 0) {
    throw std::runtime_error(fmt::format("Couldn't poll I2C slave: {}", errno));
  }
  view = static_cast<AppView>(data[0]);
  spdlog::info("View after poll: {}", view);
}

AppView AppState::get_view() { return view; }